
// Matrix Pallette
static const int MAX_MATRICES = 26;
float4x3	g_WorldMatrixArray[MAX_MATRICES] : WORLDMATRIXARRAY;
float4x4	g_ViewProj : VIEWPROJECTION;
float4x4	g_WorldITXf : WorldInverseTranspose;
float4x4	g_worldMatrix : WORLD;
float4x4	g_worldViewProj : WorldViewProjection;
float4x4	g_worldView : WorldView;

float3	g_lightPos;
float3	g_LightDirec:DIRECTION;		//Direction of Light
float4	g_LightDiffuse;
float4	g_LightAmbient;
float4	g_LightSpecular;
float4	g_Diffuse : DIFFUSE;	//Diffuse
float4	g_Ambient : AMBIENT;	//Ambient
float4	g_Specular : SPECULAR;
float4	g_Emissive;
float	g_Power;
float3	g_CameraPos;
float	g_fBump=1;	//Bump scale

int	g_NumBones = 2;

texture	g_Texture0;
texture	g_Texture1;


sampler TextureSampler = sampler_state
{
	Texture = <g_Texture0>;

	AddressU = Wrap;
	AddressV = Wrap;
};

sampler BumpSampler = sampler_state
{
	Texture = <g_Texture1>;

	AddressU = Wrap;
	AddressV = Wrap;
};


struct VS_OUTPUT_STANDARD
{
	float4 position : POSITION;
	float2 texCoord : TEXCOORD0;
	float3 halfVector : TEXCOORD1;
	float3 lightDir : TEXCOORD2;
	float3 normal : TEXCOORD3;
	float4 diffuse : COLOR0;
	float4 specular : COLOR1;
};


struct VS_OUTPUT_BUMP
{
	float4 position : POSITION;
	float2 texCoord : TEXCOORD0;
	float3 halfVector : TEXCOORD1;
	float3 lightDir : TEXCOORD2;
	float3 normal : TEXCOORD3;
	float3 binormal : TEXCOORD4;
	float3 tangent : TEXCOORD5;
	float4 diffuse : COLOR0;
	float4 specular : COLOR1;
};

VS_OUTPUT_STANDARD VS_standard(
	float4 Pos	: POSITION,
	float3 Normal : NORMAL,
	float2 texCoord:TEXCOORD0
){
	VS_OUTPUT_STANDARD OUT=(VS_OUTPUT_STANDARD)0;

	float4 worldPos = mul(Pos, g_worldViewProj);
	float3 viewDir = g_CameraPos - worldPos.xyz;

	OUT.position = worldPos;
	OUT.texCoord = texCoord;
	OUT.lightDir = -g_LightDirec;
	OUT.halfVector = normalize(normalize(OUT.lightDir) + normalize(viewDir));
	OUT.normal = mul(Normal, (float3x3)g_WorldITXf);
	OUT.diffuse = g_Diffuse * g_LightDiffuse;
	OUT.diffuse.a=g_Diffuse.a;
	OUT.specular = g_Specular * g_LightSpecular;

	return OUT;
}


VS_OUTPUT_BUMP VS_bump(
	float4 Pos	: POSITION,
	float3 Normal : NORMAL,
	float2 texCoord:TEXCOORD0,
	float3 Tangent	: TANGENT0,
	float4 Binormal	: BINORMAL0
)
{
	VS_OUTPUT_BUMP OUT=(VS_OUTPUT_BUMP)0;

	float4 worldPos = mul(Pos, g_worldViewProj);
	float3 viewDir = g_CameraPos - worldPos.xyz;
	OUT.binormal = normalize(mul(Binormal,g_WorldITXf).xyz);
	OUT.tangent = normalize(mul(Tangent,g_WorldITXf).xyz);
	OUT.normal = normalize(mul(Normal, (float3x3)g_WorldITXf));

	OUT.position = worldPos;
	OUT.texCoord = texCoord;
	OUT.lightDir = -g_LightDirec;
	OUT.halfVector = normalize(normalize(OUT.lightDir) + normalize(viewDir));
	OUT.diffuse = g_Diffuse * g_LightDiffuse;
	OUT.diffuse.a=g_Diffuse.a;
	OUT.specular = g_Specular * g_LightSpecular;

	return OUT;
}

//--------------
float4 PS_materials( VS_OUTPUT_STANDARD IN ) : COLOR
{
	float3 n = normalize(IN.normal);
	float3 h = normalize(IN.halfVector);
	float3 l = normalize(IN.lightDir);

	float nDotL = max(0,dot(n, l));
	float nDotH = max(0,dot(n, h));
	float power = (nDotL == 0.0f) ? 0.0f : pow(nDotH, g_Power);

	float4 color = saturate((g_Ambient * g_LightAmbient) + (IN.diffuse * nDotL) + g_Emissive) + (IN.specular * power);
	color.a=IN.diffuse.a;
	return color;
}

float4 PS_texture( VS_OUTPUT_STANDARD IN ) : COLOR
{
	float3 n = normalize(IN.normal);
	float3 h = normalize(IN.halfVector);
	float3 l = normalize(IN.lightDir);

	float nDotL = saturate(dot(n, l));
	float nDotH = saturate(dot(n, h));
	float power = (nDotL == 0.0f) ? 0.0f : pow(nDotH, g_Power);

	float4 color = saturate((g_Ambient * g_LightAmbient) + (IN.diffuse * nDotL) + g_Emissive) + (IN.specular * power);
	color.a=IN.diffuse.a;

	return color * tex2D(TextureSampler, IN.texCoord);
}

float4 PS_bump( VS_OUTPUT_BUMP IN ) : COLOR
{
	float3 bump = g_fBump * (tex2D(BumpSampler,IN.texCoord).rgb - float3(0.5,0.5,0.5));
	float3 n = normalize(IN.normal + bump.x*IN.tangent + bump.y*IN.binormal);
	float3 h = normalize(IN.halfVector);
	float3 l = normalize(IN.lightDir);

	float nDotL = saturate(dot(n, l));
	float nDotH = saturate(dot(n, h));
	float power = (nDotL == 0.0f) ? 0.0f : pow(nDotH, g_Power);

	float4 color = saturate((g_Ambient * g_LightAmbient) + (IN.diffuse * nDotL) + g_Emissive) + (IN.specular * power);
	color.a=IN.diffuse.a;
	color *= tex2D(TextureSampler, IN.texCoord);
	return color;
}


///////////////////////////////////////////////////////
struct VS_INPUT_BONED_STANDARD
{
	float4 Pos				: POSITION;
	float4 BlendWeights		: BLENDWEIGHT;
	float4 BlendIndices		: BLENDINDICES;
	float2 Tex0				: TEXCOORD0;
	float3 Normal			: NORMAL;
};

struct VS_INPUT_BONED_BUMP
{
	float4 Pos				: POSITION;
	float4 BlendWeights		: BLENDWEIGHT;
	float4 BlendIndices		: BLENDINDICES;
	float3 Normal			: NORMAL;
	float2 Tex0				: TEXCOORD0;
	float3 Tangent			: TANGENT0;
	float4 Binormal			: BINORMAL0;
};

VS_OUTPUT_STANDARD VS_Boned_Standard(VS_INPUT_BONED_STANDARD IN, uniform int NumBones)
{
	VS_OUTPUT_STANDARD OUT=(VS_OUTPUT_STANDARD)0;
	// Compensate for lack of UBYTE4 on Geforce3
	int4 IndexVector = D3DCOLORtoUBYTE4(IN.BlendIndices);

	// cast the vectors to arrays for use in the for loop below
	float BlendWeightsArray[4] = (float[4])IN.BlendWeights;
	int   IndexArray[4]        = (int[4])IndexVector;

	// calculate the pos/normal using the "normal" weights 
	//        and accumulate the weights to calculate the last weight
	float3	Normal = 0.0f;
	float	LastWeight = 0.0f;
	float3	Pos = 0.0f;
	int nb=NumBones-1;
	for (int iBone = 0; iBone < nb; iBone++){
		LastWeight = LastWeight + BlendWeightsArray[iBone];

		Pos += mul(IN.Pos, g_WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
		Normal += mul(IN.Normal, g_WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
	}
	LastWeight = 1.0f - LastWeight;

	// Now that we have the calculated weight, add in the final influence
	Pos += (mul(IN.Pos, g_WorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight);
	Normal += (mul(IN.Normal, g_WorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 

	// transform position from world space into view and then projection space
	OUT.position = mul(float4(Pos, 1.0f), g_ViewProj);

	// normalize normals
	OUT.normal = normalize(Normal);

	// copy the input texture coordinate through
	OUT.texCoord  = IN.Tex0.xy;

	OUT.lightDir = -g_LightDirec;

	float3 viewDir = g_CameraPos - OUT.position;
	OUT.halfVector = normalize(normalize(OUT.lightDir) + normalize(viewDir));
	OUT.diffuse = g_Diffuse * g_LightDiffuse;
	OUT.diffuse.w = g_Diffuse.w;
	OUT.specular = g_Specular * g_LightSpecular;

	return OUT;
}


VS_OUTPUT_BUMP VS_Boned_Bump(VS_INPUT_BONED_BUMP IN, uniform int NumBones)
{
	VS_OUTPUT_BUMP OUT=(VS_OUTPUT_BUMP)0;
	// Compensate for lack of UBYTE4 on Geforce3
	int4 IndexVector = D3DCOLORtoUBYTE4(IN.BlendIndices);

	// cast the vectors to arrays for use in the for loop below
	float BlendWeightsArray[4] = (float[4])IN.BlendWeights;
	int   IndexArray[4]        = (int[4])IndexVector;

	// calculate the pos/normal using the "normal" weights 
	//        and accumulate the weights to calculate the last weight
	float3	Normal = 0.0f;
	float3	Binormal = 0.0f;
	float3	Tangent = 0.0f;
	float	LastWeight = 0.0f;
	float3	Pos = 0.0f;
	float3x3 mat=(float3x3)g_WorldITXf;
	for (int iBone = 0; iBone < NumBones-1; iBone++){
		LastWeight = LastWeight + BlendWeightsArray[iBone];

		Pos += mul(IN.Pos, g_WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
		Normal += mul(IN.Normal, g_WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
		Binormal += mul(IN.Binormal, g_WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
		Tangent += mul(IN.Tangent, g_WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
	}
	LastWeight = 1.0f - LastWeight;

	// Now that we have the calculated weight, add in the final influence
	Pos += (mul(IN.Pos, g_WorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight);
	Normal += (mul(IN.Normal, g_WorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 
	Binormal += (mul(IN.Binormal, g_WorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 
	Tangent += (mul(IN.Tangent, g_WorldMatrixArray[IndexArray[NumBones-1]]) * LastWeight); 

	// transform position from world space into view and then projection space
	OUT.position = mul(float4(Pos, 1.0f), g_ViewProj);

	// normalize normals
	OUT.normal = normalize(mul(Normal,mat));
	OUT.binormal = normalize(mul(Binormal,mat));
	OUT.tangent = normalize(mul(Tangent,mat));

	// copy the input texture coordinate through
	OUT.texCoord  = IN.Tex0.xy;

	OUT.lightDir = -g_LightDirec;

	float3 viewDir = g_CameraPos - OUT.position;
	OUT.halfVector = normalize(normalize(OUT.lightDir) + normalize(viewDir));
	OUT.diffuse = g_Diffuse * g_LightDiffuse;
	OUT.diffuse.w = g_Diffuse.w;
	OUT.specular = g_Specular * g_LightSpecular;

	return OUT;
}


//////////////////////////////////////
// Techniques specs follow
//////////////////////////////////////

//----static
technique tech_materials
{
	pass p0
	{
		VertexShader = compile vs_2_0 VS_standard();
		PixelShader  = compile ps_2_0 PS_materials();
	}
}

technique tech_texture
{
	pass p0
	{
		VertexShader = compile vs_2_0 VS_standard();
		PixelShader  = compile ps_2_0 PS_texture();
	}
}

technique tech_normal
{
	pass p0
	{
		VertexShader = compile vs_2_0 VS_bump();
		PixelShader  = compile ps_2_0 PS_bump();
	}
}


//----boned

VertexShader vsArray[4] = {
	compile vs_2_0 VS_Boned_Standard(1), 
	compile vs_2_0 VS_Boned_Standard(2),
	compile vs_2_0 VS_Boned_Standard(3),
	compile vs_2_0 VS_Boned_Standard(4)
};

VertexShader vsArrayBump[4] = {
	compile vs_2_0 VS_Boned_Bump(1), 
	compile vs_2_0 VS_Boned_Bump(2),
	compile vs_2_0 VS_Boned_Bump(3),
	compile vs_2_0 VS_Boned_Bump(4)
};


technique tech_materials_boned
{
	pass p0
	{
		VertexShader = (vsArray[g_NumBones]);
		PixelShader  = compile ps_2_0 PS_materials();
	}
}

technique tech_texture_boned
{
	pass p0
	{
		VertexShader = (vsArray[g_NumBones]);
		PixelShader  = compile ps_2_0 PS_texture();
	}
}

technique tech_normal_boned
{
	pass p0
	{
		VertexShader = (vsArrayBump[g_NumBones]);
		PixelShader  = compile ps_2_0 PS_bump();
	}
}
